@@ -30,8 +30,12 @@ usages, including as a basic unordered MultiDict. Special
 thanks to `Mark Williams`_ for all his help.
 
 .. [1] As of 2015, `basic dicts on PyPy are ordered
-   <http://morepypy.blogspot.com/2015/01/faster-more-memory-efficient-and-more.html>`_.
+   <http://morepypy.blogspot.com/2015/01/faster-more-memory-efficient-and-more.html>`_,
+   and as of December 2017, `basic dicts in CPython 3 are now ordered
+   <https://mail.python.org/pipermail/python-dev/2017-December/151283.html>`_, as
+   well.
 .. _Mark Williams: https://github.com/markrwilliams
+
 """
 
 from collections import KeysView, ValuesView, ItemsView
@@ -51,7 +55,7 @@ except ImportError:
 PREV, NEXT, KEY, VALUE, SPREV, SNEXT = range(6)
 
 
-__all__ = ['MultiDict', 'OMD', 'OrderedMultiDict']
+__all__ = ['MultiDict', 'OMD', 'OrderedMultiDict', 'OneToOne']
 
 try:
     profile
@@ -69,7 +73,7 @@ class OrderedMultiDict(dict):
       * stacking data from multiple dictionaries in a non-destructive way
 
     The OrderedMultiDict constructor is identical to the built-in
-    :class:`dict`, and overall the API is constitutes an intuitive
+    :class:`dict`, and overall the API constitutes an intuitive
     superset of the built-in type:
 
     >>> omd = OrderedMultiDict()
@@ -693,4 +697,135 @@ class FastIterOrderedMultiDict(OrderedMultiDict):
             yield curr[KEY]
             curr = curr[PREV]
 
+
+_SELF_INIT_MARKER = object()
+
+
+class OneToOne(dict):
+    """Implements a one-to-one mapping dictionary. In addition to
+    inheriting from and behaving exactly like the builtin
+    :class:`dict`, all values are automatically added as keys on a
+    reverse mapping, available as the `inv` attribute. This
+    arrangement keeps key and value namespaces distinct.
+
+    Basic operations are intuitive:
+
+    >>> oto = OneToOne({'a': 1, 'b': 2})
+    >>> print(oto['a'])
+    1
+    >>> print(oto.inv[1])
+    a
+    >>> len(oto)
+    2
+
+    Overwrites happens in both directions:
+
+    >>> oto.inv[1] = 'c'
+    >>> print(oto.get('a'))
+    None
+    >>> len(oto)
+    2
+
+    """
+    __slots__ = ('inv')
+
+    def __init__(self, *a, **kw):
+        if a and a[0] is _SELF_INIT_MARKER:
+            self.inv = a[1]
+            dict.__init__(self, [(v, k) for k, v in self.inv.items()])
+        else:
+            dict.__init__(self, *a, **kw)
+            self.inv = self.__class__(_SELF_INIT_MARKER, self)
+
+    def __setitem__(self, key, val):
+        hash(val)  # ensure val is a valid key
+        if key in self:
+            dict.__delitem__(self.inv, self[key])
+        if val in self.inv:
+            del self.inv[val]
+        dict.__setitem__(self, key, val)
+        dict.__setitem__(self.inv, val, key)
+
+    def __delitem__(self, key):
+        dict.__delitem__(self.inv, self[key])
+        dict.__delitem__(self, key)
+
+    def clear(self):
+        dict.clear(self)
+        dict.clear(self.inv)
+
+    def copy(self):
+        return self.__class__(self)
+
+    def pop(self, key, default=_MISSING):
+        if key in self:
+            dict.__delitem__(self.inv, self[key])
+            return dict.pop(self, key)
+        if default is not _MISSING:
+            return default
+        raise KeyError()
+
+    def popitem(self):
+        key, val = dict.popitem(self)
+        dict.__delitem__(self.inv, val)
+        return key, val
+
+    def setdefault(self, key, default=None):
+        if key not in self:
+            self[key] = default
+        return self[key]
+
+    def update(self, dict_or_iterable, **kw):
+        if isinstance(dict_or_iterable, dict):
+            for val in dict_or_iterable.values():
+                hash(val)
+                keys_vals = list(dict_or_iterable.items())
+        else:
+            for key, val in dict_or_iterable:
+                hash(key)
+                hash(val)
+                keys_vals = list(dict_or_iterable)
+        for val in kw.values():
+            hash(val)
+        keys_vals.extend(kw.items())
+        for key, val in keys_vals:
+            self[key] = val
+
+    def __repr__(self):
+        cn = self.__class__.__name__
+        dict_repr = dict.__repr__(self)
+        return "%s(%s)" % (cn, dict_repr)
+
+
+def subdict(d, keep=None, drop=None):
+    """Compute the "subdictionary" of a dict, *d*.
+
+    A subdict is to a dict what a subset is a to set. If _A_ is a
+    subdict of _B_, that means that all keys of _A_ are present in
+    _B_.
+
+    Returns a new dict with any keys in *drop* removed, and any keys
+    in *keep* still present, provided they were in the original
+    dict. *keep* defaults to all keys, *drop* defaults to empty, so
+    without one of these arguments, calling this function is
+    equivalent to calling ``dict()``.
+
+    >>> from pprint import pprint as pp
+    >>> pp(subdict({'a': 1, 'b': 2}))
+    {'a': 1, 'b': 2}
+    >>> subdict({'a': 1, 'b': 2, 'c': 3}, drop=['b', 'c'])
+    {'a': 1}
+    >>> pp(subdict({'a': 1, 'b': 2, 'c': 3}, keep=['a', 'c']))
+    {'a': 1, 'c': 3}
+
+    """
+    if keep is None:
+        keep = d.keys()
+    if drop is None:
+        drop = []
+
+    keys = set(keep) - set(drop)
+
+    return dict([(k, v) for k, v in d.items() if k in keys])
+
 # end dictutils.py
